(function ($) {

	var mainInit = false;
	var isMediaLibrary = false;
	var currentColId = 0;
	var mainUiElement = null;
	var mutationObserver = null;
	var hierarchy = [];

	function onClick(element) {
		$('#wplr_col_id_selector')[0].value = element ? element.wp_col_id : '-1';
		$('#wplr_col_id_selector')[0].dispatchEvent(new Event('change'));
		$('#post-query-submit').click();
	}

	function buildLevel(nodes, level = 0) {
		let data = null;
		if (!level) {
			data = $('<ul class="tree" style="margin: 8px 5px;"></ul>');
			var clickable = $('<a style="text-decoration: none; cursor: pointer;" href="#"> ' + (currentColId === -1 ? ('<b>All Media</b>') : 'All Media') + '</a>');
			clickable.click(function() { onClick() });
			var menu = $('<li></li>');
			menu.append('<span class="dashicons dashicons-admin-media"></span>');
			menu.append(clickable);
			data.append(menu);
		}
		else {
			data = $('<ul class="tree" style="border-left: 1px solid gray; margin-left: 2px; padding: 5px 5px 0px 10px;"></ul>');
		}

		nodes.forEach(function (element) {
			if (element.is_folder) {
				let li = $('<li class="folder" style="margin-bottom: 5px;"></li>');
				let label = $('<span class="label"><span class="dashicons dashicons-category"></span> ' + element.name + '</span>').appendTo(li);

				if (element.children.length > 0) {
					label.wrap('<a href="#" class="fold">');
					let newData = buildLevel(element.children, level + 1);
					li.append(newData);

				} else { // Empty folder
					li.addClass('empty');
				}
				data.append(li);
			}
			else {
				let li = $('<li class="item" style="margin-bottom: 5px;"></li>');
				let isCurrent = currentColId === element.wp_col_id;
				if (isCurrent) li.addClass('current');
				let icon = $('<span class="dashicons dashicons-format-gallery"></span>');
				li.append(icon);
				icon.click(function() { alert('[gallery wplr-collection="' + element.wp_col_id + '"]'); });
				var clickable = $('<a style="text-decoration: none; cursor: pointer; href="#"> ' + (isCurrent ? ('<b>' + element.name + '</b>') : element.name)
					+ '</a>');
				clickable.click(function() { onClick(element) });
				li.append(clickable);
				data.append(li);
			}
		});

		if (!level) {
			// Enable Folding/Unfolding
			data.find('.folder > a.fold').on('click', function (ev) {
				ev.preventDefault();
				var folder = $(this).parent();
				folder.toggleClass('unfolded');
			});
			// Reveal the current item if it was hidden inside a folder
			data.find('.folder .item.current')
				.parents('.folder')
				.addClass('unfolded');
		}
		return data;
	}

	function buildUI() {
		// Create the sidebar (hierarchy of folders and collections)
		var wrap = $('<div id="wplr-menu" class="wp-filter">').hide();

		// Restore the saved state of the UI
		if (readCookie('wplr-sync.explorer.minimized', 0)) wrap.addClass('minimized');

		// Titlebar
		var titlebar = $('<div class="titlebar">').appendTo(wrap);
		var title = $('<div class="title">Lightroom</div>').appendTo(titlebar);

		// Minimizer
		var button = $('<a href="#" class="minimize"><i class="dashicons dashicons-arrow-left">').appendTo(titlebar);
		button.on('click', function (ev) {
			ev.preventDefault();
			wrap.toggleClass('minimized');

			// Save the state to the cookie
			writeCookie('wplr-sync.explorer.minimized', wrap.hasClass('minimized') ? 1 : 0, 3); // Preserve for 3 days
		});

		var content = $('<div class="content">');
		var data = buildLevel(hierarchy, 0);
		content.append(data);
		wrap.append(content);

		// Create the new container
		var newMediaLib = $('<div style="display: flex;"></div>');

		// Add the sidebar to the container, and move the listing handled by WP to the container
		newMediaLib.append(wrap);
		$(mainUiElement).css('flex', '1');
		$(mainUiElement).parent().append(newMediaLib);
		$(mainUiElement).detach().appendTo(newMediaLib);

		// Fade in the results
		wrap.fadeIn();
		$(mainUiElement).fadeIn();
	}

	function buildUIforModal() {
		currentColId = parseInt(lrsync_library.col_id);
		// Create the sidebar (hierarchy of folders and collections)
		var leftSide = $('<div id="wplr-menu">');
		leftSide.append('<div style="border-bottom: 1px solid #e5e4e5; border-top: 1px solid #e5e4e5; padding: 10px; background: #fafafa;">WP/LR Sync</div>');
		var list = $('<ul style="margin: 5px 8px;"></ul>');
		var data = buildLevel(hierarchy, 0);
		list.append(data);
		leftSide.append(list);
		$('.attachments-browser > div:eq(1)').after(leftSide);
		$('.attachments-browser ul').css('left', '200px');
	}

	function combineAllClasses(childNodes) {
		childNodes = Array.from(childNodes);
		var classes = [];
		for (let node of childNodes) {
			if (node.className && node.className !== '')
				classes += node.className + '|';
			if (node.childNodes) {
				var newClasses = combineAllClasses(node.childNodes);
				classes += newClasses;
			}
		}
		return classes;
	}

	var mutationObserver = function (mutationsList, observer) {
		let addedNodes = mutationsList.map(x => x.addedNodes);
		let allClasses = addedNodes.map(x => combineAllClasses(x)).filter(x => x.length > 0).join(',');

		// Media Library
		if (isMediaLibrary && allClasses.includes('attachments-browser')) {
			mainUiElement = '.media-frame-content';
			// One time is enough, no need to listen to the DOM events
			mutationObserver.disconnect();
			buildUI();
		}
		// Select or Upload Media Modal
		else if (!isMediaLibrary && allClasses.includes('attachments-browser')) {
			// If not sortable, it is 'Add to Gallery' and not 'Edit Gallery", so we display the menu
			let isAdd = jQuery('.ui-sortable-disabled').length;
			if (isAdd) {
				mainUiElement = '.media-frame-content';
				buildUIforModal();
			}
		}
	}

	function addTaxonomyToFilters() {

		if (!wp.media || !wp.media.view)
			return;

		// Create a new AttachmentFilters
		var MediaLibraryTaxonomyFilter = wp.media.view.AttachmentFilters.extend({
			id: 'wplr_col_id_selector',
			createFilters: function() {
				var filters = {};
				filters['all'] = { text: 'All Media', props: { col_id: -1 }, priority: 10 };
				filters[-1] = { text: 'All Media', props: { col_id: -1 }, priority: 10 };
				_.each( collections || {}, function( value, index ) {
					filters[value.wp_col_id] = {
						text: value.name,
						props: { col_id: value.wp_col_id }
					};
				});
				this.filters = filters;
			}
		});
		// Extend and override wp.media.view.AttachmentsBrowser to include the new filter
		var AttachmentsBrowser = wp.media.view.AttachmentsBrowser;
		wp.media.view.AttachmentsBrowser = wp.media.view.AttachmentsBrowser.extend({
			createToolbar: function() {
				// Load original toolbar
				AttachmentsBrowser.prototype.createToolbar.call( this );
				this.toolbar.set( 'MediaLibraryTaxonomyFilter', new MediaLibraryTaxonomyFilter({
					controller: this.controller,
					model:  this.collection.props,
					priority: -75
				}).render() );
			}
		});
	}

	// Re-create the hierarchy of folders and collections
	function feedFolder(collections, folderId = null) {
			var data = [];
			var folderCols = collections.filter(x => x.wp_folder_id === folderId);
			for (var collection of folderCols) {
				if (collection.is_folder)
					collection.children = feedFolder(collections, collection.wp_col_id);
				data.push(collection);
			}
			return data;
		}

	/**
	 * Stores a value with the specified key to the cookie
	 * @param string name The key
	 * @param any value The value
	 * @param number days The lifespan
	 */
	function writeCookie(name, value, days) {
		let date = new Date();
		date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
		document.cookie = name + '=' + value + ';expires=' + date.toUTCString();
	}

	/**
	 * Retrieves a value with the specified key from the cookie
	 * @param string name The key
	 * @param any def (Optional) The default value to be returned if the key didn't exist
	 * @return The retrieved value
	 */
	function readCookie(name, def = '') {
		let decoded = decodeURIComponent(document.cookie)
		let data = decoded.split(';');
		for (let item of data) {
			item = item.trim();
			if (!item.startsWith(name + '=')) continue;
			let r = item.substring(name.length + 1);
			// Type Casting
			switch (typeof def) {
			case 'string':
				return String(r);
			case 'number':
				return Number(r);
			case 'boolean':
				return Boolean(r);
			}
			return r;
		}
		return def;
	}

	function startup() {

		currentColId = parseInt(lrsync_library.col_id);
		isMediaLibrary = lrsync_library.isMediaLibrary === '1';
		collections = lrsync_library.hierarchy.map(x => { return {
			name: x.name,
			level: parseInt(x.level),
			is_folder: x.is_folder === '1',
			wp_col_id: parseInt(x.wp_col_id),
			wp_folder_id: x.wp_folder_id ? parseInt(x.wp_folder_id) : null
		}});
		hierarchy = feedFolder(collections);

		if (!wp) {
			return;
		}

		// Media Library: List Mode
		if ($('#posts-filter').length > 0) {
			mainUiElement = '#posts-filter';
			buildUI();
			return;
		}

		// Override the default filters for Grid mode
		addTaxonomyToFilters();

		// Let's wait for something to happen
		if ($('.wp-core-ui').length > 0) {
			mutationObserver = new MutationObserver(mutationObserver);
			mutationObserver.observe(document.getElementsByClassName('wp-admin')[0], { childList: true, subtree: true });
		}
		mainInit = true;
	}

	$(document).ready(startup);

})(jQuery);
