<?php

class Meow_WPLR_Sync_Explorer {

	public $screen;

	public function __construct() {

		$mode = get_option( 'wplr_library_show_hierarchy', 'none' );

		// FILTERS IN THE MEDIA LIBRARY
		if ( $mode === 'filters' || $mode === 'explorer' ) {
			add_action( 'pre_get_posts', array( $this, 'media_filters' ) );
			add_action( 'restrict_manage_posts', array( $this, 'media_dropdown' ) );
		}

		// UI WITH HIERARCHY IN THE MEDIA LIBRARY
		if ( $mode === 'explorer' ) {
			add_action( 'admin_head', array( $this, 'admin_head' ) );
			add_action( 'admin_footer', array( $this, 'admin_footer' ) );
			add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
		}
	}

	/*
		FILTERS IN THE MEDIA LIBRARY
	*/

	function media_filters( $q ) {
		$col_id = filter_input( INPUT_GET, 'col_id', FILTER_SANITIZE_STRING );
		if ( isset( $_POST['query'] ) && isset( $_POST['query']['col_id'] ) )
			$col_id = (int)( $_POST['query']['col_id'] );
		if ( $col_id < 1 ) {
			return;
		}
		global $wplr;
		$pids = $wplr->get_media_from_collection( $col_id );
		if ( !empty( $pids ) ) {
			$q->set( 'post__in', $pids );
			$q->set( 'orderby', 'post__in' );
		}
		else
			$q->set( 'p', -1 );
		return $q;
	}

	function media_dropdown() {
		if ( !$this->shouldBeOverriden() )
			return;
		$wp_parent_folder = null;
		global $wpdb, $wplr;
		$tbl_c = $wpdb->prefix . 'lrsync_collections';
		$collections = $wpdb->get_results( "SELECT * FROM $tbl_c WHERE wp_folder_id IS NULL ORDER BY lr_col_id", OBJECT );
		$col_id = filter_input( INPUT_GET, 'col_id', FILTER_SANITIZE_STRING );
		echo '<a class="dashicons-before dashicons-camera" style="top: 5px; position: relative; margin: 5px;"></a>
			<select name="col_id" id="wplr_col_id_selector" class="postform">
			<option value="-1" selected="selected">All Media</option>';
		$collections = $wplr->read_collections_recursively();
		foreach ( $collections as $c ) {
			echo '<option class="level-' . $c['level'] . '"' .
				( $c['is_folder'] ? ' disabled' : '' ) . ' value="' .
				$c['wp_col_id'] . '" ' . selected( $c['wp_col_id'], $col_id ) . '>' .
				str_pad( "", $c['level'] * 6 * 3, "&nbsp;" ) . $c['name'] .
				'</option>';
		}
		echo '</select>';
	}

	/*
		UI WITH HIERARCHY IN THE MEDIA LIBRARY
	*/

	function admin_head() {
		if ( !$this->shouldBeOverriden() )
			return;

		?>
		<style>
			#wplr-menu {
 				position: relative;
 				display: flex;
 				flex-direction: column;
				width: 200px;
 				margin-right: 10px;
 				padding: 0px;
				overflow: hidden;

				transition-property: width;
				transition-duration: .25s;
				transition-timing-function: ease-out;
			}
			#wplr-menu.minimized {
				width: 41px;
				height: 54px;
				margin-right: -2px;
			}
			.media-modal-content #wplr-menu {
				position: absolute;
				top: 50px;
				left: 0;
				bottom: 0;
				overflow: auto;
				border-right: 1px solid #e8e8e8;
			}

			#wplr-menu .titlebar,
			#wplr-menu .content {
				box-sizing: border-box;
			}

			#wplr-menu .titlebar {
				position: relative;
				flex: 0 0 auto;
				overflow: hidden;
				background-color: #0073aa;
			}
			#wplr-menu .titlebar .title {
				overflow: hidden;
				white-space: nowrap;
				padding: 17px 12px;
				color: #f1f1f1;
				font-size: 16px;
			}
			#wplr-menu .titlebar a.minimize {
				position: absolute;
				top: 0;
				right: 0;
				width: 39px;
				height: 100%;
				background-color: #0073aa;
				color: white;
			}
			#wplr-menu .titlebar a.minimize:focus {
				outline: none;
				box-shadow: unset;
			}
			#wplr-menu .titlebar a.minimize:hover {
				background-color: #008acc; /* Slightly brighter */
			}
			#wplr-menu .titlebar a.minimize i {
				/* Centering */
				position: absolute;
				top: 0; bottom: 0;
				left: 0; right: 0;
				margin: auto;

				opacity: .75;
				transition: transform .25s ease-out;
			}
			#wplr-menu .titlebar a.minimize:hover i {
				opacity: 1;
			}
			#wplr-menu.minimized .titlebar a.minimize i {
				transform: rotate(180deg);
			}

			#wplr-menu .content {
				flex: 1 1 auto;
				width: 200px;
				padding: 5px 8px;
				overflow: auto;
			}
			#wplr-menu.minimized .content {
				display: none;
			}

			#wplr-menu .folder > a.fold {
				text-decoration: none;
			}
			#wplr-menu .folder > .tree {
				display: none;
			}
			#wplr-menu .folder.unfolded > .tree {
				display: block;
			}
		</style>
		<?php
		ob_start( array( $this, "hide_media_gallery" ) );
	}

	function hide_media_gallery( $buffer ) {
		if ( !isset( $buffer ) || trim( $buffer ) === '' )
			return $buffer;
		$buffer = str_replace( 'id="posts-filter"', 'id="posts-filter" style="display: none;"', $buffer );
		return $buffer;
	}

	function admin_footer() {
		if ( !$this->shouldBeOverriden() )
			return;
		@ob_end_flush();
	}

	function shouldBeOverriden() {
		$should = false;
		$screen = get_current_screen();
		if ($screen->base === 'upload')
			return true;
		if ( isset( $_SERVER['QUERY_STRING'] ) ) {
			if ( preg_match( '/action=edit/', $_SERVER['QUERY_STRING'] ) )
				return true;
		}
	}

	function admin_enqueue_scripts() {
		if ( !$this->shouldBeOverriden() )
			return;
		global $wplr_version;
		global $wplr;
		$screen = get_current_screen();
		$collections = $wplr->read_collections_recursively();
		wp_enqueue_script( 'lrsync_library', plugins_url( '/explorer.js', __FILE__ ),
			array( 'jquery'), $wplr_version, false );
		wp_localize_script( 'lrsync_library', 'lrsync_library', array(
			'hierarchy' => $collections,
			'isMediaLibrary' => $screen->base === 'upload',
			'col_id' => (int)filter_input( INPUT_GET, 'col_id', FILTER_SANITIZE_STRING )
		) );
	}

}

?>
