<?php
/*
Plugin Name: WP/LR Sync
Plugin URI: https://meowapps.com
Description: Synchronize and maintain your photos, collections, keywords and metadata between Lightroom and WordPress.
Version: 5.1.5
Author: Jordy Meow
Author URI: https://meowapps.com
Text Domain: wplr-sync
Domain Path: /languages

Originally developed for two of my websites:
- Jordy Meow (https://offbeatjapan.org)
- Haikyo (https://haikyo.org)
*/

include "lrsync_lrinfo.php";
include "lrsync_core.php";

if ( !get_option( 'wplr_upgrade_5_0' ) ) {
	function wplr_upgrade_5_0_admin_notices() {
		echo '<div class="notice notice-error"><h2>WP/LR Sync 5: Big Changes!</h2>';
		echo '<p>Do not worry, those are very positive changes, the plugin is getting much better! <b>Did just install WP/LR Sync for the first time?</b> Ignore this. <b>Have you already hidden this message before?</b> Sorry. This time, the message will go away forever, I promise :) <b>Were you using a version of WP/LR Sync prior to 5?</b> You must update the plugin on the Lightroom side, and use a token instead of the login/password. Is is all explained here: <a target="_blank" href="https://meowapps.com/wplr-sync-5-0/">WP/LR Sync 5.0</a>. Thank you :)</p>';
		echo '<p>
			<form method="post" action="">
				<input type="hidden" name="wplr_upgrade_5_0" value="true">
				<input type="submit" name="submit" id="submit" class="button" value="Understood. Got this. Hide this forever!">
			</form>
		</p>
		';
		echo '</div>';
	}
	if ( isset( $_POST['wplr_upgrade_5_0'] ) ) {
		update_option( 'wplr_upgrade_5_0', true, false );
	}
	else
		add_action( 'admin_notices', 'wplr_upgrade_5_0_admin_notices' );
}

function meow_wplrsync_activate() {
	global $wpdb;
	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
	$charset_collate = $wpdb->get_charset_collate();
	$tbl_lrsync = $wpdb->prefix . "lrsync";
	$sql = "CREATE TABLE $tbl_lrsync (
		id BIGINT(20) NOT NULL AUTO_INCREMENT,
		wp_id BIGINT(20) NULL,
		lr_id BIGINT(20) NULL,
		lr_file TINYTEXT NULL,
		lastsync DATETIME NULL,
		UNIQUE KEY id (id)
	) " . $charset_collate . ";";
	dbDelta($sql);
	$tbl_collections = $wpdb->prefix . "lrsync_collections";
	$isNewToCollections = ( $wpdb->get_var("SHOW TABLES LIKE '$tbl_collections'") != $tbl_collections );
	$sql = "CREATE TABLE $tbl_collections (
		wp_col_id BIGINT(20) NOT NULL AUTO_INCREMENT,
		lr_col_id BIGINT(20) NULL,
		wp_folder_id BIGINT(20) NULL,
		name TINYTEXT NULL,
		is_folder TINYINT(1) NOT NULL DEFAULT 0,
		lastsync DATETIME NULL,
		UNIQUE KEY id (wp_col_id)
	) " . $charset_collate . ";";
	dbDelta($sql);
	$tbl_relations = $wpdb->prefix . "lrsync_relations";
	$sql = "CREATE TABLE $tbl_relations (
		wp_col_id BIGINT(20) NULL,
		wp_id BIGINT(20) NULL,
		sort INT(11) DEFAULT 0,
		UNIQUE KEY (wp_col_id, wp_id)
	) " . $charset_collate . ";";
	dbDelta($sql);
	$tbl_meta = $wpdb->prefix . "lrsync_meta";
	$isNewToCollections = ( $wpdb->get_var("SHOW TABLES LIKE '$tbl_meta'") != $tbl_meta );
	$sql = "CREATE TABLE $tbl_meta (
		meta_id BIGINT(20) NOT NULL AUTO_INCREMENT,
		name TINYTEXT NULL,
		id BIGINT(20) NULL,
		value LONGTEXT NULL,
		UNIQUE KEY id (meta_id)
	) " . $charset_collate . ";";
	dbDelta($sql);
	// If this install is new to Collections, insert all the linked
	// media in the -1 collection (default one)
	if ( $isNewToCollections ) {
		$wpdb->query( "INSERT INTO $tbl_relations (wp_col_id, wp_id, sort) SELECT -1, wp_id, 0 FROM $tbl_lrsync" );
	}
}

function meow_wplrsync_uninstall() {
	// Better to avoid removing the table...
	global $wpdb;
	$tbl_col = $wpdb->prefix . 'lrsync_collections';
	$tbl_r = $wpdb->prefix . 'lrsync_relations';
	$tbl_m = $wpdb->prefix . 'lrsync_meta';
	$tbl_lr = $wpdb->prefix . 'lrsync';
	$wpdb->query( "DROP TABLE IF EXISTS $tbl_col" );
	$wpdb->query( "DROP TABLE IF EXISTS $tbl_r" );
	$wpdb->query( "DROP TABLE IF EXISTS $tbl_lr" );
	$wpdb->query( "DROP TABLE IF EXISTS $tbl_m" );
}

register_activation_hook( WP_PLUGIN_DIR . '/wplr-sync/wp-lrsync.php', 'meow_wplrsync_activate' );
//register_uninstall_hook( WP_PLUGIN_DIR . '/wplr-sync/wp-lrsync.php', 'meow_wplrsync_uninstall' );

global $wplr, $wplr_version;
$wplr_version = '5.1.5';
$wplr = new Meow_WPLR_Sync_Core( $wplr_version );

if ( is_admin() ) {
	include "lrsync_admin.php";
	$wplr_admin = new Meow_WPLR_Sync_Admin( 'lrsync', __FILE__, 'wplr-sync' );
}

include "lrsync_api.php";
new Meow_WPLR_Sync_API();

if ( get_option( "wplr_public_api", true ) ) {
	require_once( dirname(__FILE__) . '/public_api/public_api.php' );
	new Meow_WPLR_Sync_Public_API();
}

?>
